#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2020 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import argparse
import sys
import os
from fosslight_util.help import print_package_version
from fosslight_binary._help import print_help_msg
from fosslight_binary.binary_analysis import find_binaries
from fosslight_util.timer_thread import TimerThread

_PKG_NAME = "fosslight_binary"


def main():
    global windows
    path_to_find_bin = ""
    output_dir = ""
    format = ""
    db_url = ""

    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('-h', '--help', action='store_true', required=False)
    parser.add_argument('-v', '--version', action='store_true', required=False)
    parser.add_argument('-p', '--path', type=str, required=False)
    parser.add_argument('-o', '--output', type=str, required=False)
    parser.add_argument('-d', '--dburl', type=str, default='', required=False)
    parser.add_argument('-f', '--format', type=str, required=False)
    parser.add_argument('--notice', action='store_true', required=False)

    try:
        args = parser.parse_args()
    except Exception as ex:
        print(ex)
        sys.exit(1)

    if args.help:  # -h option
        print_help_msg()

    if args.version:    # -v option
        print_package_version(_PKG_NAME, "FOSSLight Binary Scanner Version:")
        sys.exit(0)

    if args.path:   # -p option
        path_to_find_bin = args.path
    else:
        path_to_find_bin = os.getcwd()

    if args.output:  # -o option
        output_dir = args.output

    if args.dburl:  # -d option
        db_url = args.dburl

    if args.format:  # -f option
        format = args.format

    if args.notice:  # --notice option
        try:
            base_path = sys._MEIPASS
        except Exception:
            base_path = os.path.dirname(__file__)

        data_path = os.path.join(base_path, 'LICENSES')
        print(f"*** {_PKG_NAME} open source license notice ***")
        for ff in os.listdir(data_path):
            f = open(os.path.join(data_path, ff), 'r', encoding='utf8')
            print(f.read())
        sys.exit(0)

    timer = TimerThread()
    timer.setDaemon(True)
    timer.start()

    find_binaries(path_to_find_bin, output_dir, format, db_url)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2020 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
from fosslight_util.help import PrintHelpMsg

_HELP_MESSAGE_BINARY = """
    Usage: fosslight_bin [option1] <arg1> [option2] <arg2>...

    After extracting the binaries, the open source and license information of the saved binaries are retrieved by comparing the similarity
    with the binaries stored in the Binary DB (FOSSLight > Binary DB) with the Binary's TLSH (Trend micro Locality Sensitive Hash).

    Options:
        -p <binary_path>\t\t    Path to analyze binaries (Default: current directory)
        -h\t\t\t\t    Print help message
        -v\t\t\t\t    Print FOSSLight Binary Scanner version
        -o <output_path>\t\t    Output path
        \t\t\t\t    (If you want to generate the specific file name, add the output path with file name.)
        -f <format>\t\t\t    Output file format (excel, csv, opossum, yaml)
        -d <db_url>\t\t\t    DB Connection(format :'postgresql://username:password@host:port/database_name')
        --notice\t\t\t    Print the open source license notice text."""


def print_help_msg():
    helpMsg = PrintHelpMsg(_HELP_MESSAGE_BINARY)
    helpMsg.print_help_msg(True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2020 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import hashlib
import tlsh
from io import open

_EXCLUDE_TRUE_VALUE = "Exclude"
_TLSH_CHECKSUM_NULL = "0"


class OssItem:
    name = ""
    version = ""
    license = ""
    dl_url = ""
    comment = ""
    exclude = False

    def __init__(self, name, version, license, dl_url=""):
        self.name = name
        self.version = version
        self.license = license
        self.dl_url = dl_url
        self.exclude = False
        self.comment = ""

    def set_comment(self, value):
        self.comment += value

    def set_exclude(self, value):
        self.exclude = value

    def get_comment(self):
        return self.comment


class VulnerabilityItem:
    file_path = ""
    vul_id = ""
    nvd_url = ""

    def __init__(self, file_path, id, url):
        self.file_path = file_path
        self.vul_id = id
        self.nvd_url = url


class BinaryItem:
    bin_name = ""
    binary_name_without_path = ""
    binary_strip_root = ""  # Value of binary name column
    tlsh = _TLSH_CHECKSUM_NULL
    checksum = _TLSH_CHECKSUM_NULL
    oss_items = []
    vulnerability_items = []
    exclude = False
    comment = ""
    found_in_db = False

    def __init__(self, value):
        self.exclude = False
        self.binary_strip_root = ""
        self.checksum = _TLSH_CHECKSUM_NULL
        self.tlsh = _TLSH_CHECKSUM_NULL
        self.oss_items = []
        self.vulnerability_items = []
        self.binary_name_without_path = ""
        self.set_bin_name(value)

    def __del__(self):
        pass

    def set_oss_items(self, new_oss_list, exclude_old=False, exclude_msg=""):
        if exclude_old:
            for old_oss in self.oss_items:
                old_oss.set_exclude(True)
                old_oss.set_comment(exclude_msg)
        # Append New input OSS
        self.oss_items.extend(new_oss_list)

    def set_vulnerability_items(self, vul_list):
        if vul_list is not None:
            self.vulnerability_items.extend(vul_list)

    def get_vulnerability_items(self):
        nvd_url = [vul_item.nvd_url for vul_item in self.vulnerability_items]
        return ", ".join(nvd_url)

    def set_commnet(self, value):
        self.comment = value

    def set_bin_name(self, value):
        self.bin_name = value

    def set_exclude(self, value):
        self.exclude = value

    def set_checksum(self, value):
        self.checksum = value

    def set_tlsh(self, value):
        self.tlsh = value

    def get_comment(self):
        return self.comment

    def get_print_binary_only(self):
        return (self.binary_strip_root + "\t" + self.checksum + "\t" + self.tlsh)

    def get_oss_report(self):
        if len(self.oss_items) > 0:
            for oss in self.oss_items:
                exclude = _EXCLUDE_TRUE_VALUE if (self.exclude or oss.exclude) else ""
                nvd_url = self.get_vulnerability_items()
                yield from [self.binary_strip_root, oss.name, oss.version,
                            oss.license, oss.dl_url, '', '', exclude, oss.comment, nvd_url]
        else:
            exclude = _EXCLUDE_TRUE_VALUE if self.exclude else ""
            yield from [self.binary_strip_root, '',
                        '', '', '', '', '', exclude, '']

    def set_checksum_tlsh(self):
        self.checksum, self.tlsh, error, msg = get_checksum_and_tlsh(
            self.bin_name)
        return error, msg


def get_checksum_and_tlsh(bin_with_path):
    checksum_value = _TLSH_CHECKSUM_NULL
    tlsh_value = _TLSH_CHECKSUM_NULL
    error_msg = ""
    error = False
    try:
        f = open(bin_with_path, "rb")
        byte = f.read()
        sha1_hash = hashlib.sha1(byte)
        checksum_value = str(sha1_hash.hexdigest())
        try:
            tlsh_value = str(tlsh.hash(byte))
        except:
            tlsh_value = _TLSH_CHECKSUM_NULL
        f.close()
    except Exception as ex:
        error_msg = f"(Error) Get_checksum, tlsh: {ex}"
        error = True
    return checksum_value, tlsh_value, error, error_msg



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2022 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import logging
import json
import os
import sys
import fosslight_util.constant as constant
from ._binary import BinaryItem, OssItem, VulnerabilityItem
from dependency_check import run as dependency_check_run


logger = logging.getLogger(constant.LOGGER_NAME)


def run_analysis(params, func):
    try:
        sys.argv = params
        func()
    except SystemExit:
        pass
    except Exception as ex:
        logger.error(f"Run Analysis : {ex}")


def get_oss_ver(version_info):
    oss_version = ""
    if version_info.get('source') == 'central':
        if version_info.get('name') == 'version':
            oss_version = version_info.get('value')
    elif version_info.get('source') == 'pom':
        if version_info.get('name') == 'version':
            oss_version = version_info.get('value')
    elif version_info.get('source', "").lower() == 'manifest':
        if version_info.get('name') == 'Implementation-Version' or version_info.get('name') == 'Bundle-Version':
            oss_version = version_info.get('value')
    return oss_version


def get_oss_lic_in_jar(data):
    license = ""
    license_raw = str(data.get("license"))
    split_lic = license_raw.split(':')[0]

    # Not NoneType but string 'None'
    if license_raw == "None":
        license = ""
    else:
        if not split_lic.startswith('http'):
            license = split_lic.replace(',', '')
        else:
            license = license_raw

    return license


def merge_binary_list(owasp_items, vulnerability_items, bin_list):
    not_found_bin = []

    # key : file_path / value : oss_list for one binary
    for key, value in owasp_items.items():
        found = False
        for bin in bin_list:
            if bin.binary_strip_root == key:
                bin.set_oss_items(value, False)
                if vulnerability_items is not None:
                    bin.set_vulnerability_items(vulnerability_items.get(key))
                found = True
                break

        if not found:
            bin_item = BinaryItem(os.path.abspath(key))
            bin_item.binary_name_without_path = os.path.basename(key)
            bin_item.binary_strip_root = key
            bin_item.set_oss_items(value)
            not_found_bin.append(bin_item)

    bin_list += not_found_bin
    return bin_list


def get_vulnerability_info(file_with_path, vulnerability, vulnerability_items, remove_vulnerability_items):
    if vulnerability:
        try:
            for vul_info in vulnerability:
                vul_id = ""
                nvd_url = ""
                for key, val in vul_info.items():
                    if key == 'id':
                        vul_id = val
                    elif key == 'url':
                        nvd_url = val

                vul_item = VulnerabilityItem(file_with_path, vul_id, nvd_url)

                remove_vulnerability_items = vulnerability_items.get(file_with_path)
                if remove_vulnerability_items:
                    remove_vulnerability_items.append(vul_item)
                else:
                    vulnerability_items[file_with_path] = [vul_item]
        except Exception as ex:
            logger.info(f"Error to get vul_id and nvd_url: {ex}")

    return vulnerability_items


def get_oss_groupid(evidence_info):
    oss_groupid = ""
    # First, Get groupid from Central, else get it from pom
    if evidence_info.get('source') == 'central':
        if evidence_info.get('name') == 'groupid':
            oss_groupid = evidence_info.get('value')
    elif evidence_info.get('source') == 'pom':
        if evidence_info.get('name') == 'groupid':
            oss_groupid = evidence_info.get('value')
    return oss_groupid


def get_oss_artifactid(evidence_info):
    oss_artifactid = ""
    # Get OSS Info from POM
    if evidence_info.get('source') == 'pom':
        if evidence_info.get('name') == 'artifactid':
            oss_artifactid = evidence_info.get('value')
    return oss_artifactid


def get_oss_dl_url(evidence_info):
    oss_dl_url = ""
    if evidence_info.get('name') == 'url':
        oss_dl_url = evidence_info.get('value')
    return oss_dl_url


def get_oss_info_from_pkg_info(pkg_info):
    oss_name = ""
    oss_version = ""

    try:
        if pkg_info.get('id') != "":
            # Get OSS Name
            if pkg_info.get('id').startswith('pkg:maven'):
                # ex, pkg:maven/com.hankcs/aho-corasick-double-array-trie@1.2.3
                oss_name = pkg_info.get('id').split('@')[0]
                oss_name = f"{oss_name.split('/')[-2]}:{oss_name.split('/')[-1]}"
            elif pkg_info.get('id').startswith('pkg:npm'):
                # ex, pkg:npm/cryptiles@0.2.2
                oss_name = pkg_info.get('id').split('@')[0]
                oss_name = oss_name.replace('pkg:npm', 'npm')
                oss_name = oss_name.replace('/', ':')
            else:
                oss_name = pkg_info.get('id').split('@')[0]
                oss_name = oss_name.split('/')[-1]
            # Get OSS Version
            oss_version = pkg_info.get('id').split('@')[1]
    except Exception as ex:
        logger.debug(f"Error to get value for oss name and version: {ex}")
    return oss_name, oss_version


def analyze_jar_file(path_to_find_bin):
    remove_owasp_item = []
    owasp_items = {}
    remove_vulnerability_items = []
    vulnerability_items = {}
    success = True
    json_file = ""

    command = ['dependency-check', '--scan', f'{path_to_find_bin}', '--out', f'{path_to_find_bin}',
               '--disableArchive', '--disableAssembly', '--disableRetireJS', '--disableNodeJS',
               '--disableNodeAudit', '--disableNugetconf', '--disableNuspec', '--disableOpenSSL',
               '--disableOssIndex', '--disableBundleAudit', '--cveValidForHours', '24', '-f', 'JSON']
    try:
        run_analysis(command, dependency_check_run)
    except Exception as ex:
        logger.info(f"Error to analyze .jar file - OSS information for .jar file isn't included in report.\n {ex}")
        success = False
        return owasp_items, vulnerability_items, success

    try:
        json_file = os.path.join(path_to_find_bin, 'dependency-check-report.json')
        with open(json_file, 'r') as f:
            jar_contents = json.load(f)
    except Exception as ex:
        logger.debug(f"Error to read dependency-check-report.json file : {ex}")
        success = False
        return owasp_items, vulnerability_items, success

    dependencies = jar_contents.get("dependencies")
    try:
        for val in dependencies:
            bin_with_path = ""
            oss_name = ""
            oss_ver = ""
            oss_artifactid = ""
            oss_groupid = ""
            oss_dl_url = ""
            oss_license = get_oss_lic_in_jar(val)
            oss_name_found = False

            all_evidence = val.get("evidenceCollected", {})
            vulnerability = val.get("vulnerabilityIds", [])
            all_pkg_info = val.get("packages", [])

            vendor_evidences = all_evidence.get('vendorEvidence', [])
            version_evidences = all_evidence.get('versionEvidence', [])

            # Check if the file is .jar file
            # Even if the oss info is from pom.xml in jar file, the file name will be .jar file.
            # But the oss info from pom.xml could be different from .jar file.
            bin_with_path = val.get("filePath")
            if not bin_with_path.endswith('.jar'):
                bin_with_path = bin_with_path.split('.jar')[0] + '.jar'

            file_with_path = os.path.relpath(bin_with_path, path_to_find_bin)

            # First, Get OSS Name and Version info from pkg_info
            for pkg_info in all_pkg_info:
                oss_name, oss_ver = get_oss_info_from_pkg_info(pkg_info)

            if oss_name == "" and oss_ver == "":
                # If can't find name and version, Find thoes in vendorEvidence and versionEvidence .
                # Get Version info from versionEvidence
                for version_info in version_evidences:
                    oss_ver = get_oss_ver(version_info)

                # Get Artifact ID, Group ID, OSS Name from vendorEvidence
                for vendor_info in vendor_evidences:
                    if oss_groupid == "":
                        oss_groupid = get_oss_groupid(vendor_info)
                    if oss_artifactid == "":
                        oss_artifactid = get_oss_artifactid(vendor_info)
                    if oss_dl_url == "":
                        oss_dl_url = get_oss_dl_url(vendor_info)
                    # Combine groupid and artifactid
                    if oss_artifactid != "" and oss_groupid != "":
                        oss_name = f"{oss_groupid}:{oss_artifactid}"
                        oss_name_found = True
                    # If oss_name is found, break
                    if oss_name_found:
                        break
            else:
                # Get only dl_url from vendorEvidence
                for vendor_info in vendor_evidences:
                    if oss_dl_url == "":
                        oss_dl_url = get_oss_dl_url(vendor_info)

            # Get Vulnerability Info.
            vulnerability_items = get_vulnerability_info(file_with_path, vulnerability, vulnerability_items, remove_vulnerability_items)

            if oss_name != "" or oss_ver != "" or oss_license != "" or oss_dl_url != "":
                oss = OssItem(oss_name, oss_ver, oss_license, oss_dl_url)
                oss.set_comment("OWASP Result. ")

                remove_owasp_item = owasp_items.get(file_with_path)
                if remove_owasp_item:
                    remove_owasp_item.append(oss)
                else:
                    owasp_items[file_with_path] = [oss]
    except Exception as ex:
        logger.debug(f"Error to get depency Info in jar_contets: {ex}")
        success = False

    try:
        if os.path.isfile(json_file):
            os.remove(json_file)
    except Exception as ex:
        logger.debug(f"Error - There is no .json file : {ex}")

    return owasp_items, vulnerability_items, success



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# FOSSLight Binary analysis script
# Copyright (c) 2020 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import os
import sys
from datetime import datetime
from binaryornot.check import is_binary
import magic
import logging
import yaml
import stat
from fosslight_util.set_log import init_log
import fosslight_util.constant as constant
from fosslight_util.write_txt import write_txt_file
from fosslight_util.output_format import check_output_format, write_output_file
from ._binary_dao import get_oss_info_from_db
from ._binary import BinaryItem
from ._jar_analysis import analyze_jar_file, merge_binary_list

_PKG_NAME = "fosslight_binary"
logger = logging.getLogger(constant.LOGGER_NAME)

_REMOVE_FILE_EXTENSION = ['png', 'gif', 'jpg', 'bmp', 'jpeg', 'qm', 'xlsx', 'pdf', 'ico', 'pptx', 'jfif', 'docx',
                          'doc', 'whl', 'xls', 'xlsm', 'ppt', 'mp4', 'pyc', 'plist']
_REMOVE_FILE_COMMAND_RESULT = [
    'data', 'timezone data', 'apple binary property list']
INCLUDE_FILE_COMMAND_RESULT = ['current ar archive']
_EXCLUDE_FILE = ['fosslight_bin', 'fosslight_bin.exe']
_EXCLUDE_DIR = ["test", "tests", "doc", "docs"]
_EXCLUDE_DIR = [os.path.sep + dir_name + os.path.sep for dir_name in _EXCLUDE_DIR]
_EXCLUDE_DIR.append("/.")
_REMOVE_DIR = ['.git']
_REMOVE_DIR = [os.path.sep + dir_name + os.path.sep for dir_name in _REMOVE_DIR]
_error_logs = []
_root_path = ""
_start_time = ""
windows = False
BYTES = 2048

JAR_VUL_HEADER = {'BIN_FL_Binary': ['ID', 'Source Name or Path', 'OSS Name',
                                    'OSS Version', 'License', 'Download Location',
                                    'Homepage', 'Copyright Text', 'Exclude',
                                    'Comment', 'Vulnerability Link']}


def init(path_to_find_bin, output_file_name, format):
    global _root_path, logger, _start_time

    _json_ext = ".json"
    _start_time = datetime.now().strftime('%y%m%d_%H%M')
    _result_log = {
        "Tool Info": _PKG_NAME
    }

    _root_path = path_to_find_bin
    if not path_to_find_bin.endswith(os.path.sep):
        _root_path += os.path.sep

    success, msg, output_path, output_file, output_extension = check_output_format(output_file_name, format)
    if success:
        if output_path == "":
            output_path = os.getcwd()
        else:
            output_path = os.path.abspath(output_path)

        if output_file != "":
            result_report = output_file
            bin_txt_file = f"{output_file}.txt"
        else:
            if output_extension == _json_ext:
                result_report = f"fosslight_opossum_{_start_time}"
            else:
                result_report = f"fosslight_report_{_start_time}"
            bin_txt_file = f"fosslight_binary_{_start_time}.txt"

        result_report = os.path.join(output_path, result_report)
        binary_txt_file = os.path.join(output_path, bin_txt_file)
    else:
        logger.error(f"Format error - {msg}")
        sys.exit(1)

    log_file = os.path.join(output_path, f"fosslight_log_{_start_time}.txt")
    logger, _result_log = init_log(log_file, True, logging.INFO, logging.DEBUG, _PKG_NAME, path_to_find_bin)

    if not success:
        error_occured(error_msg=msg,
                      result_log=_result_log,
                      exit=True)
    return _result_log, result_report, binary_txt_file, output_extension


def get_file_list(path_to_find):
    bin_list = []
    file_cnt = 0
    found_jar = False

    for root, dirs, files in os.walk(path_to_find):
        for file in files:
            file_lower_case = file.lower()
            extension = file_lower_case.split(".")[-1]

            if extension == 'jar':
                found_jar = True

            directory = root + os.path.sep
            dir_path = directory.replace(_root_path, '', 1).lower()
            dir_path = os.path.sep + dir_path + os.path.sep
            if any(dir_name in dir_path for dir_name in _REMOVE_DIR):
                continue

            bin_with_path = os.path.join(root, file)
            bin_item = BinaryItem(bin_with_path)
            bin_item.binary_name_without_path = file
            bin_item.binary_strip_root = bin_with_path.replace(
                _root_path, '', 1)

            if any(dir_name in dir_path for dir_name in _EXCLUDE_DIR):
                bin_item.set_exclude(True)
            elif file.lower() in _EXCLUDE_FILE:
                bin_item.set_exclude(True)
            bin_list.append(bin_item)
            file_cnt += 1

    return file_cnt, bin_list, found_jar


def find_binaries(path_to_find_bin, output_dir, format, dburl=""):

    _result_log, result_report, binary_txt_file, output_extension = init(
        path_to_find_bin, output_dir, format)

    total_bin_cnt = 0
    total_file_cnt = 0
    db_loaded_cnt = 0
    success_to_write = False
    writing_msg = ""
    extended_header = {}
    content_list = []
    result_file = ""

    if not os.path.isdir(path_to_find_bin):
        error_occured(error_msg=f"Can't find the directory : {path_to_find_bin}",
                      result_log=_result_log,
                      exit=True)
    try:
        total_file_cnt, file_list, found_jar = get_file_list(path_to_find_bin)
        return_list = list(return_bin_only(file_list))
    except Exception as ex:
        error_occured(error_msg=f"Failed to check whether it is binary or not : {ex}",
                      result_log=_result_log,
                      exit=True)
    total_bin_cnt = len(return_list)
    try:
        # Run OWASP Dependency-check
        if found_jar:
            logger.info("Run OWASP Dependency-check to analyze .jar file")
            owasp_items, vulnerability_items, success = analyze_jar_file(path_to_find_bin)
            if success:
                return_list = merge_binary_list(owasp_items, vulnerability_items, return_list)
                extended_header = JAR_VUL_HEADER
            else:
                logger.warning("Could not find OSS information for some jar files.")

        return_list, db_loaded_cnt = get_oss_info_from_db(return_list, dburl)
        return_list = sorted(return_list, key=lambda row: (row.bin_name))

        if return_list:
            str_files = (x.get_print_binary_only() for x in return_list)
            success, error = write_txt_file(binary_txt_file,
                                            "Binary\tsha1sum\ttlsh\n" + '\n'.join(str_files))

            if success:
                _result_log["FOSSLight binary.txt"] = binary_txt_file
            else:
                error_occured(error_msg=error, exit=False)

        sheet_list = {}
        content_list = [list(item.get_oss_report()) for item in return_list]
        sheet_list["BIN_FL_Binary"] = content_list

        success_to_write, writing_msg, result_file = write_output_file(result_report, output_extension, sheet_list, extended_header)
    except Exception as ex:
        error_occured(error_msg=str(ex), exit=False)

    try:
        if success_to_write:
            if result_file:
                logger.info(f"Output file :{result_file}")
            else:
                logger.warning(f"{writing_msg}")
        else:
            logger.error(f"Fail to generate result file.:{writing_msg}")

        print_result_log(success=True, result_log=_result_log,
                         file_cnt=str(total_file_cnt),
                         bin_file_cnt=str(total_bin_cnt),
                         auto_bin_cnt=str(db_loaded_cnt))
    except Exception as ex:
        error_occured(error_msg=f"Print log : {ex}", exit=False)

    return success_to_write, content_list


def return_bin_only(file_list, need_checksum_tlsh=True):
    for file_item in file_list:
        if check_binary(file_item.bin_name):
            if need_checksum_tlsh:
                error, error_msg = file_item.set_checksum_tlsh()
                if error:
                    error_occured(error_msg=error_msg, exit=False)
            yield file_item


def check_binary(file_with_path):
    is_bin_confirmed = False
    file = os.path.basename(file_with_path)
    extension = os.path.splitext(file)[1][1:]
    if not os.path.islink(file_with_path) and extension.lower() not in _REMOVE_FILE_EXTENSION:
        if stat.S_ISFIFO(os.stat(file_with_path).st_mode):
            return False
        file_command_result = ""
        file_command_failed = False
        try:
            file_command_result = magic.from_file(file_with_path)
        except Exception:
            file_command_failed = True
        if file_command_failed:
            try:
                file_command_result = magic.from_buffer(open(file_with_path).read(BYTES))
            except Exception as ex:
                logger.debug(f"Failed to check file type:{file_with_path}, {ex}")

        if file_command_result:
            file_command_result = file_command_result.lower()
            if any(file_command_result.startswith(x) for x in _REMOVE_FILE_COMMAND_RESULT):
                return False
            if any(file_command_result.startswith(x) for x in INCLUDE_FILE_COMMAND_RESULT):
                is_bin_confirmed = True
        if is_binary(file_with_path):
            is_bin_confirmed = True
    return is_bin_confirmed


def error_occured(error_msg, exit=False, result_log={}):
    global _error_logs
    _error_logs.append(error_msg)
    if exit:
        print_result_log(success=False, result_log=result_log)
        sys.exit()


def print_result_log(success=True, result_log={}, file_cnt="", bin_file_cnt="", auto_bin_cnt=""):

    if "Running time" in result_log:
        start_time = result_log["Running time"]
    else:
        start_time = _start_time
    result_log["Running time"] = start_time + " ~ " + \
        datetime.now().strftime('%Y%m%d_%H%M%S')
    result_log["Execution result"] = 'Success' if success else 'Error occurred'
    result_log["Binaries / Scanned files"] = f"{bin_file_cnt}/{file_cnt}"
    result_log["Identified in Binary DB / Binaries"] = f"{auto_bin_cnt}/{bin_file_cnt}"
    if len(_error_logs) > 0:
        result_log["Error Log"] = _error_logs
        if success:
            result_log["Execution result"] += " but it has minor errors"
    try:
        _str_final_result_log = yaml.safe_dump(result_log, allow_unicode=True, sort_keys=True)
        logger.info(_str_final_result_log)
    except Exception as ex:
        logger.warning(f"Error to print final log: {ex}")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
from codecs import open
import os
import shutil
from setuptools import setup, find_packages

with open('README.md', 'r', 'utf-8') as f:
    readme = f.read()

with open('requirements.txt', 'r', 'utf-8') as f:
    install_requires = f.read().splitlines()

_PACKAEG_NAME = 'fosslight_binary'
_LICENSE_FILE = 'LICENSE'
_LICENSE_DIR = 'LICENSES'

if __name__ == "__main__":
    dest_path = os.path.join('src', _PACKAEG_NAME, _LICENSE_DIR)
    try:
        if not os.path.exists(dest_path):
            os.mkdir(dest_path)
        if os.path.isfile(_LICENSE_FILE):
            shutil.copy(_LICENSE_FILE, dest_path)
        if os.path.isdir(_LICENSE_DIR):
            license_f = [f_name for f_name in os.listdir(_LICENSE_DIR) if f_name.upper().startswith(_LICENSE_FILE)]
            for lic_f in license_f:
                shutil.copy(os.path.join(_LICENSE_DIR, lic_f), dest_path)
    except Exception as e:
        print(f'Warning: Fail to copy the license text: {e}')

    setup(
        name=_PACKAEG_NAME,
        version='4.1.18',
        package_dir={"": "src"},
        packages=find_packages(where='src'),
        description='FOSSLight Binary Scanner',
        long_description=readme,
        long_description_content_type='text/markdown',
        license='Apache-2.0',
        author='LG Electronics',
        url='https://github.com/fosslight/fosslight_binary_scanner',
        download_url='https://github.com/fosslight/fosslight_binary_scanner',
        classifiers=['License :: OSI Approved :: Apache Software License',
                     "Programming Language :: Python :: 3",
                     "Programming Language :: Python :: 3.6",
                     "Programming Language :: Python :: 3.7",
                     "Programming Language :: Python :: 3.8",
                     "Programming Language :: Python :: 3.9", ],
        install_requires=install_requires,
        extras_require={
            ':sys_platform == "win32"': [
                'python-magic-bin'
            ],
            ':"darwin" in sys_platform': [
                'python-magic'
            ],
            ':"linux" in sys_platform': [
                'python-magic'
            ],
        },
        package_data={_PACKAEG_NAME: [os.path.join(_LICENSE_DIR, '*')]},
        include_package_data=True,
        entry_points={
            "console_scripts": [
                "binary_analysis = fosslight_binary.cli:main",
                "fosslight_bin = fosslight_binary.cli:main",
                "fosslight_binary = fosslight_binary.cli:main",
            ]
        }
    )
    shutil.rmtree(dest_path, ignore_errors=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2020 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import tlsh
import logging
import psycopg2
import pandas as pd
from urllib.parse import urlparse
from ._binary import _TLSH_CHECKSUM_NULL, OssItem
import fosslight_util.constant as constant

columns = ['filename', 'pathname', 'checksum', 'tlshchecksum', 'ossname',
           'ossversion', 'license', 'platformname',
           'platformversion']
conn = ""
cur = ""
logger = logging.getLogger(constant.LOGGER_NAME)


def get_oss_info_from_db(bin_info_list, dburl=""):
    _cnt_auto_identified = 0
    conn_str = get_connection_string(dburl)
    connect_to_lge_bin_db(conn_str)

    if conn != "" and cur != "":
        for item in bin_info_list:
            bin_oss_items = []
            tlsh_value = item.tlsh
            checksum_value = item.checksum
            bin_file_name = item.binary_name_without_path

            df_result = get_oss_info_by_tlsh_and_filename(
                bin_file_name, checksum_value, tlsh_value)
            if df_result is not None and len(df_result) > 0:
                _cnt_auto_identified += 1
                for idx, row in df_result.iterrows():
                    oss_from_db = OssItem(
                        row['ossname'], row['ossversion'], row['license'])
                    oss_from_db.set_comment("Binary DB Result")
                    bin_oss_items.append(oss_from_db)

                if bin_oss_items:
                    item.found_in_db = True
                    item.set_oss_items(bin_oss_items, True, "Excluded due to Binary DB.")
    disconnect_lge_bin_db()
    return bin_info_list, _cnt_auto_identified


def get_connection_string(dburl):
    # dburl format : 'postgresql://username:password@host:port/database_name'
    connection_string = ""
    user_dburl = True
    if dburl == "" or dburl is None:
        user_dburl = False
        dburl = "postgresql://bin_analysis_script_user:script_123@bat.lge.com:5432/bat"
    try:
        if user_dburl:
            logger.debug("DB URL:" + dburl)
        dbc = urlparse(dburl)
        connection_string = "dbname={dbname} user={user} host={host} password={password} port={port}" \
            .format(dbname=dbc.path.lstrip('/'),
                    user=dbc.username,
                    host=dbc.hostname,
                    password=dbc.password,
                    port=dbc.port)
    except Exception as ex:
        if user_dburl:
            logger.warning(f"(Minor) Failed to parsing db url : {ex}")

    return connection_string


def get_oss_info_by_tlsh_and_filename(file_name, checksum_value, tlsh_value):
    sql_statement = "SELECT filename,pathname,checksum,tlshchecksum,ossname,ossversion,\
                    license,platformname,platformversion FROM lgematching "
    sql_statement_checksum = " WHERE filename='{fname}' AND checksum='{checksum}';".format(fname=file_name,
                                                                                           checksum=checksum_value)  # Checking checksum first.
    sql_statement_filename = "SELECT DISTINCT ON (tlshchecksum) tlshchecksum FROM lgematching WHERE filename='{fname}';".format(
        fname=file_name)  # For getting tlsh values of file.

    final_result_item = ""

    df_result = get_list_by_using_query(
        sql_statement + sql_statement_checksum, columns)
    # Found a file with the same checksum.
    if df_result is not None and len(df_result) > 0:
        final_result_item = df_result
    else:
        # Match tlsh and fileName
        df_result = get_list_by_using_query(
            sql_statement_filename, ['tlshchecksum'])
        if df_result is None or len(df_result) <= 0:
            final_result_item = ""
        elif tlsh_value == _TLSH_CHECKSUM_NULL:  # Couldn't get the tlsh of a file.
            final_result_item = ""
        else:
            matched_tlsh = ""
            matched_tlsh_diff = -1
            for row in df_result.tlshchecksum:
                try:
                    if row != _TLSH_CHECKSUM_NULL:
                        tlsh_diff = tlsh.diff(row, tlsh_value)
                        if tlsh_diff <= 120:  # MATCHED
                            if (matched_tlsh_diff < 0) or (tlsh_diff < matched_tlsh_diff):
                                matched_tlsh_diff = tlsh_diff
                                matched_tlsh = row
                except Exception as ex:
                    logger.warning(f"* (Minor) Error_tlsh_comparison: {ex}")
            if matched_tlsh != "":
                final_result_item = get_list_by_using_query(
                    sql_statement + " WHERE filename='{fname}' AND tlshchecksum='{tlsh}';".format(fname=file_name,
                                                                                                  tlsh=matched_tlsh),
                    columns)

    return final_result_item


def get_list_by_using_query(sql_query, columns):
    result_rows = ""  # DataFrame
    cur.execute(sql_query)
    rows = cur.fetchall()

    if rows is not None and len(rows) > 0:
        result_rows = pd.DataFrame(data=rows, columns=columns)
    return result_rows


def disconnect_lge_bin_db():
    global conn, cur
    try:
        cur.close()
        conn.close()
    except:
        pass


def connect_to_lge_bin_db(connection_string):
    global conn, cur

    try:
        conn = psycopg2.connect(connection_string)
        cur = conn.cursor()
    except Exception as ex:
        logger.debug(f"(Minor) Can't connect to Binary DB. : {ex}")
        conn = ""
        cur = ""
